//File name converter and file management functions
//Luke Lenhart (2007)
//See /docs/License.txt for details on how this code may be used.

#include "File.h"
#include <algorithm>

namespace MPMA
{
	//remove rediculousness from a filename
	void Filename::PurifyFilename()
	{
		//strip out adjacent / or \ characters
		std::string newFname;
		newFname.reserve(fname.size());
		bool prevSlash=false;
		for (std::string::iterator i=fname.begin(); i!=fname.end(); ++i)
		{
			bool curSlash=*i=='/' || *i=='\\';
			
			if (curSlash)
			{
				if (!prevSlash)
					newFname.push_back(*i);
			}
			else
			{
				newFname.push_back(*i);
			}
			
			prevSlash=curSlash;
		}
		
		fname=newFname;
	}
	
	// --
	
	//ctor
	Directory::Directory()
	{
	}
	Directory::Directory(const Filename &initialPath)
	{
		path=initialPath;
	}
	
	//change to a local-relative directory
	void Directory::StepOut()
	{
		
		if (path.empty())  //if no path is set (so "current")
		{
			path="..";
		}
		else if (path.size()>=2 && path[path.size()-1]=='.' && path[path.size()-2]=='.') //if the last path chunk is ".."
		{
			path+="/..";
		}
		else //normal case, strip off the last path chunk
		{
			while (!path.empty())
			{
				char c=path[path.size()-1];
				path.resize(path.size()-1);
				if (c=='/' || c=='\\')
					break;
			}
		}
	}
	void Directory::StepInto(const std::string &subdir)
	{
		if (!path.empty())
			path+="/"+subdir;
		else
			path=subdir;
	}
}
