//!\file Setup.h Initialization and configuration options for different parts of the framework.
//Luke Lenhart, 2007
//See /docs/License.txt for details on how this code may be used.

#pragma once

// -- -- -- Framework Setup -- -- --
//! The Base MPMA Framework
namespace MPMA
{
	//!\brief You MUST declare this (and only 1 of it) at the start of the scope in which you want the framework to be used.
	//!It will handle all setup and cleanup required.
	//!Most likely you want to do this at the start of main().
	class InitAndShutdown
	{
	public:
		InitAndShutdown();
		~InitAndShutdown();
	};
	
	//Internal framework use only: registers init and shutdown callbacks
	void Internal_AddInitCallback(void (*pfunc)(), int priority);
	void Internal_AddShutdownCallback(void (*pfunc)(), int priority);
}


// -- -- -- Options -- -- --

// -- Profiler --

//!If defined, allows profiling of release builds.
#define PROFILE_RELEASE


// -- MemMan --

//!If defined, memory tracking operations will be performed in release builds.
#define ENABLE_MEMMAN_TRACE_IN_RELEASE
#undef ENABLE_MEMMAN_TRACE_IN_RELEASE

//!If defined, the call stack will be saved for every allocation.
#define SAVE_ALLOC_CALL_STACK

//!If defined, information about freed memory will be saved for later checks.  This essentially creates a leak since old information is never freed.
#define MEM_TRACK_OLD_FREE
#undef MEM_TRACK_OLD_FREE

//If defined, tracks the maximum memory size allocated through the manager.
//#define TRACK_MAX_MEM


// -- Debug --

//!Toggle this to enable/disable retrieving the call stack (not a cheap operation)
#define DEBUG_CALLSTACK_ENABLED

//!Toggle this off to disable all output to the reporter system
#define DEBUGROUTER_ENABLED

//!The amount of data a reporter can buffer before having to wait on its worker thread to catch up
#define DEBUGROUTER_BUFFER_SIZE (8*1024)


// -- Platform-specific --

//!Allows references to StackWalk64 and symbol functions on windows, which require the dbghelp library.  See http://msdn2.microsoft.com/en-us/library/ms679294.aspx for details.
#define WIN32_ALLOW_STACK_WALK
