//file name converter and file management functions
//Luke Lenhart (2007)
//See /docs/License.txt for details on how this code may be used.

#include "../File.h"
#include "../DebugRouter.h"
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>

namespace
{
	//retrieves a list of all entries in a directory of a certain type
	std::vector<std::string> GetFilesOfType(const std::string &path, unsigned char type)
	{
		std::string openPath=path.empty()?"./":path;

		DIR *dir=opendir(openPath.c_str());
		if (dir==0)
			return std::vector<std::string>();
		else
		{
			std::vector<std::string> list;
			dirent *ent;
			while ((ent=readdir(dir))!=0)
			{
				if (ent->d_type==type && std::string(".")!=ent->d_name && std::string("..")!=ent->d_name)
					list.push_back(ent->d_name);
			}
			closedir(dir);
			
			return list;
		}
	}
}

namespace MPMA
{
	//Construct this with a non-absolute filename using either windows or linux style, to
	// generate a filename for whatever platform this is running on.
	//Filenames that begin with ~ are relative to the users home directory.
	Filename::Filename(const std::string &filename)
	{
		if (filename.empty()) return;
		
		//
		unsigned int len=filename.size();
		fname.reserve(len);
		unsigned int startChar=0;
		if (filename[0]=='\\') startChar=1;
		for (unsigned int i=startChar; i<len; ++i)
		{
			char c=filename[i];
			//if (c==':') continue; //some platforms totally choke on this char, so totally remove it to maintain consistancy
			if (c=='\\') c='/';
				fname.push_back(c);
		}
		
		PurifyFilename();
	}
	
	// --
	
	//create a directory if it does not already exist
	bool Directory::CreateDirectory()
	{
		//make parents of that dir if we need to
		Directory od(*this);
		if (od.path.length()!=0)
		{
			od.StepOut();
			if (od.path.length()!=0)
			{
				if (!od.CreateDirectory())
					return false;
			}
		}
		
		//do the final make
		int ret=mkdir(path.c_str(), S_IRWXU|S_IRWXG|S_IROTH);
		int err=errno;
		if (!(ret==0 || err==EEXIST))
		{
			ErrorReport()<<"Create directory failed on \""<<path<<"\": "<<strerror(errno)<<"\n";
			return false;
		}
		
		return true;
	}
		
	//retrieves a list of all files in this directory
	std::vector<std::string> Directory::GetFiles() const
	{
		return GetFilesOfType(path, DT_REG);
	}
	
	//retrieves a list of all directories in this directory (no . or ..)
	std::vector<std::string> Directory::GetSubDirectories() const
	{
		return GetFilesOfType(path, DT_DIR);
	}
}
