//Luke Lenhart, 2007
//(filename is different to work around a msvc ide bug that prevented compilation)
//See /docs/License.txt for details on how this code may be used.

#include "../Locks.h"
#include "../MemMan.h"
#include "evil_windows.h"

// -- mutex setup

namespace MPMA
{

MutexLock::MutexLock()
{
	if (mMan.IsReady())
	{
		isMemManaged=true;
		crit=new2(CRITICAL_SECTION,CRITICAL_SECTION);
	}
	else
	{
		isMemManaged=false;
		crit=new CRITICAL_SECTION;;
	}
	InitializeCriticalSection((CRITICAL_SECTION*)crit);
}

MutexLock::~MutexLock()
{
	DeleteCriticalSection((CRITICAL_SECTION*)crit);
	CRITICAL_SECTION* theCrit=(CRITICAL_SECTION*)crit;
	if (isMemManaged)
		delete2(theCrit);
	else
		delete theCrit;
	crit=0;
}


// -- mutex use

TakeMutexLock::TakeMutexLock(MutexLock &critSection):
	crit(critSection)
{
	EnterCriticalSection((CRITICAL_SECTION*)crit.crit);
}

TakeMutexLock::~TakeMutexLock()
{
	LeaveCriticalSection((CRITICAL_SECTION*)crit.crit);
}


// -- BlockingObject

struct BlockObjData
{
	HANDLE handEvent;
	std::list<HANDLE> waitingList; //things blocked waiting

	void SignalAll()
	{
		SetEvent(handEvent);

		for (std::list<HANDLE>::iterator i=waitingList.begin(); i!=waitingList.end(); ++i)
		{
			SetEvent(*i);
			CloseHandle(*i);
		}
		waitingList.clear();
	}
};

BlockingObject::BlockingObject()
{
	isSet=false;
	pdata=new2(BlockObjData,BlockObjData);
	BlockObjData *bodata=(BlockObjData*)pdata;

	bodata->handEvent=CreateEvent(0, true, true, 0);
}

BlockingObject::~BlockingObject()
{
	BlockObjData *bodata=(BlockObjData*)pdata;
	
	CloseHandle(bodata->handEvent);

	delete2(bodata);
}

//Marks the object as cleared, so nothing is blocking on it.
void BlockingObject::Clear()
{
	BlockObjData *bodata=(BlockObjData*)pdata;
	
	TakeSpinLock takeLock(lock);
	isSet=false;

	bodata->SignalAll();
}

//Marks the object as set, so any all Waits will block on it.
void BlockingObject::Set()
{
	BlockObjData *bodata=(BlockObjData*)pdata;

	TakeSpinLock takeLock(lock);
	isSet=true;

	ResetEvent(bodata->handEvent);
}

//Blocks and waits for as long as the object is set (or if time is non-max, then until that many milliseconds pass).  Returns false only on timeout.
bool BlockingObject::WaitUntilClear(unsigned int timeToWait)
{
	if (!isSet) return true;
	
	BlockObjData *bodata=(BlockObjData*)pdata;
	
	TakeSpinLock takeLock(lock, true);
	
	HANDLE newHand=0;
	takeLock.Leave();
	
	if (timeToWait==0xffffffff)
	{
		if (newHand) WaitForSingleObject(newHand, INFINITE);
		else WaitForSingleObject(bodata->handEvent, INFINITE);
		return true;
	}
	else
	{
		unsigned int ret;
		if (newHand) WaitForSingleObject(newHand, timeToWait);
		else ret=WaitForSingleObject(bodata->handEvent, timeToWait);
		return ret!=WAIT_TIMEOUT;
	}
}

};
