//tests that filename converter and file management functions work
//Luke Lenhart, 2007
//See /docs/License.txt for details on how this code may be used.

#include "../UnitTest.h"
#include <string>
#include <iostream>
#include <algorithm>
#include "base/Info.h"
#include "base/File.h"

#ifdef DECLARE_TESTS_CODE
class FilenameConversion: public UnitTest
{
public:
	bool Run()
	{
		bool passed=true;
		
		//
		std::string name1=MPMA::Filename("subdirectory/test/name.txt");
		std::string name2=MPMA::Filename("subdirectory\\test\\name.txt");
		std::string name3=MPMA::Filename("subdirectory/test\\name.txt");
		
		if (name1!=name2 || name2!=name3)
		{
			std::cout<<"slash conversion is incorrect."<<std::endl;
			passed=false;
		}
		
		//
		std::string homePathBase="~";
		std::string homePathExpanded=MPMA::Filename(homePathBase);
		std::cout<<homePathBase<<" expanded to "<<homePathExpanded<<std::endl;
		std::string pathSeperator="/";
#ifdef _WIN32
		pathSeperator="\\";
		if (homePathBase==homePathExpanded || homePathExpanded.size()<4)
		{
			std::cout<<"home directory did not expand correctly"<<std::endl;
			passed=false;
		}
#endif
		
		std::string extraPath="something";
		std::string homePlusExtra=MPMA::Filename(homePathBase+pathSeperator+extraPath);
		std::string homePlusExtra2=homePathExpanded+pathSeperator+extraPath;
		
		if (homePlusExtra!=homePlusExtra2)
		{
			std::cout<<"home with subpath did not expand correctly."<<std::endl;
			passed=false;
		}

		return passed;
	}

};
#endif
DECLARE_TEST_CASE(FilenameConversion)


#ifdef DECLARE_TESTS_CODE
class DirectoryCreate: public UnitTest
{
public:
	bool Run()
	{
		bool passed=true;
		
		MPMA::Directory d("temp/deep/directory");
		passed=d.CreateDirectory();

		return passed;
	}

};
#endif
DECLARE_TEST_CASE(DirectoryCreate)


#ifdef DECLARE_TESTS_CODE
class DirectoryBrowsing: public UnitTest
{
public:
	bool Run()
	{
		bool passed=true;

		{
			MPMA::Directory d1;
			std::vector<std::string> files=d1.GetFiles();
			if (files.empty())
			{
				std::cout<<"No files listed in (default) dir "<<(const std::string&)d1<<std::endl;
				passed=false;
			}
			
			MPMA::Directory d2(d1);
			files=d2.GetFiles();
			if (files.empty())
			{
				std::cout<<"No files in dir "<<(const std::string&)d2<<std::endl;
				passed=false;
			}
		}
		
		{
			MPMA::Directory d("~");
			d.StepOut();
			std::vector<std::string> dirs=d.GetSubDirectories();
			if (dirs.empty())
			{
				std::cout<<"No dirs in step above home dir:"<<(const std::string&)d<<std::endl;
				passed=false;
			}
			
			for (std::vector<std::string>::iterator i=dirs.begin(); i!=dirs.end(); ++i)
			{
				if (*i=="." || *i=="..")
				{
					std::cout<<"Subdirectory list contained an unexpected entry: "<<*i<<std::endl;
					passed=false;
				}
			}
		}
		
		{
			//make 2 test dirs
			MPMA::Directory d("temp/testdir1");
			d.CreateDirectory();
			d.StepOut();
			d.StepInto("testdir2");
			d.CreateDirectory();
			d.StepOut();
			
			//verify they're in dir list
			d=MPMA::Directory();
			std::vector<std::string> dirs=d.GetSubDirectories();
			if (std::find(dirs.begin(), dirs.end(), "temp")==dirs.end())
			{
				std::cout<<"Could not find directory \"temp\" inside of current directory."<<std::endl;
				passed=false;
			}
			else
			{
				d.StepInto("temp");
				std::vector<std::string> dirs=d.GetSubDirectories();
				if (std::find(dirs.begin(), dirs.end(), "testdir1")==dirs.end() || std::find(dirs.begin(), dirs.end(), "testdir2")==dirs.end())
				{
					std::cout<<"Could not find directory testdir1 and testdir2 inside of temp directory."<<std::endl;
					passed=false;
				}
			}
		}

		return passed;
	}

};
#endif
DECLARE_TEST_CASE(DirectoryBrowsing)
