//tests that MemMan is thread-safe
//Luke Lenhart, 2007
//See /docs/License.txt for details on how this code may be used.

#include "../UnitTest.h"
#include <vector>
#include "base/Thread.h"
#include "base/Locks.h"
using namespace MPMA;

#ifdef DECLARE_TESTS_CODE
class MemManSafety: public UnitTest
{
private:
	static bool badness;

public:
	bool Run()
	{
		badness=false;

		//make 10 test threads
		std::vector<Thread*> threads;
		for (int i=0; i<10; ++i)
			threads.push_back(new2(Thread(UberThread, ThreadParam(0)),Thread));
		while (!threads.empty())
		{
			delete2(threads.back());
			threads.pop_back();
		}

		//
		return !badness;
	}
	
	//thread that does some alloc and frees
	static void UberThread(Thread &me, ThreadParam param)
	{
		//main test
		for (volatile int i=0; i<3000; ++i)
		{
			if (i%5==1 || i%5==4) //single
			{
				//alloc
				volatile int *a=new2(int,int);
				*a=i;

				//twiddle our thumbs
				++i; ++i; ++i;
				--i; --i; --i;

				//check
				if (*a!=i)
				{
					badness=true;
					std::cout<<"My single int changed...!\n";
					return;
				}

				delete2(a);
			}
			else //array
			{
				int num=((i+111)*33333)%100000;
				//alloc
				volatile int *a=new2_array(int,num,int);
				
				for (int c=0; c<num; ++c)
					a[c]=i+c;

				//twiddle our thumbs
				++i; ++i; ++i;
				--i; --i; --i;

				//check
				for (int c=0; c<num; ++c)
				{
					if (a[c]!=i+c)
					{
						badness=true;
						std::cout<<"My array int changed...!\n";
						return;
					}
				}

				delete2_array(a);
			}
		}
	}
};
bool MemManSafety::badness;
#endif
DECLARE_TEST_CASE(MemManSafety)
